{
  "type": "record",
  "name": "InstitutionalMemory",
  "namespace": "com.linkedin.pegasus2avro.common",
  "doc": "Institutional memory of an entity. This is a way to link to relevant documentation and provide description of the documentation. Institutional or tribal knowledge is very important for users to leverage the entity.",
  "fields": [
    {
      "name": "elements",
      "type": {
        "type": "array",
        "items": {
          "type": "record",
          "name": "InstitutionalMemoryMetadata",
          "doc": "Metadata corresponding to a record of institutional memory.",
          "fields": [
            {
              "name": "url",
              "type": "string",
              "doc": "Link to an engineering design document or a wiki page.",
              "java": {
                "class": "com.linkedin.pegasus2avro.common.url.Url",
                "coercerClass": "com.linkedin.pegasus2avro.common.url.UrlCoercer"
              }
            },
            {
              "name": "description",
              "type": "string",
              "doc": "Description of the link."
            },
            {
              "name": "createStamp",
              "type": {
                "type": "record",
                "name": "AuditStamp",
                "doc": "Data captured on a resource/association/sub-resource level giving insight into when that resource/association/sub-resource moved into a particular lifecycle stage, and who acted to move it into that specific lifecycle stage.",
                "fields": [
                  {
                    "name": "time",
                    "type": "long",
                    "doc": "When did the resource/association/sub-resource move into the specific lifecycle stage represented by this AuditEvent."
                  },
                  {
                    "name": "actor",
                    "type": "string",
                    "doc": "The entity (e.g. a member URN) which will be credited for moving the resource/association/sub-resource into the specific lifecycle stage. It is also the one used to authorize the change.",
                    "java": {
                      "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                    }
                  },
                  {
                    "name": "impersonator",
                    "type": [
                      "null",
                      "string"
                    ],
                    "doc": "The entity (e.g. a service URN) which performs the change on behalf of the Actor and must be authorized to act as the Actor.",
                    "default": null,
                    "java": {
                      "class": "com.linkedin.pegasus2avro.common.urn.Urn"
                    }
                  },
                  {
                    "name": "message",
                    "type": [
                      "null",
                      "string"
                    ],
                    "doc": "Additional context around how DataHub was informed of the particular change. For example: was the change created by an automated process, or manually.",
                    "default": null
                  }
                ]
              },
              "doc": "Audit stamp associated with creation of this record"
            }
          ]
        }
      },
      "doc": "List of records that represent institutional memory of an entity. Each record consists of a link, description, creator and timestamps associated with that record."
    }
  ],
  "Aspect": {
    "name": "institutionalMemory"
  }
}