{
  "type": "record",
  "name": "DataHubPolicyKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "doc": "Key for a DataHub Policy",
  "fields": [
    {
      "name": "id",
      "type": "string",
      "doc": "A unique id for the DataHub access policy record. Generated on the server side at policy creation time."
    }
  ],
  "Aspect": {
    "name": "dataHubPolicyKey",
    "keyForEntity": "dataHubPolicy",
    "entityCategory": "internal",
    "entityAspects": [
      "dataHubPolicyInfo"
    ],
    "entityDoc": "DataHub Policies represent access policies granted to users or groups on metadata operations like edit, view etc."
  }
}